import urllib2
import os
from time import gmtime, strftime, time
from singlet.lens import SingleScopeLens, IconViewCategory, ListViewCategory
from cleardarksky import cleardarkskyconfig
from math import floor
import libxml2

class CleardarkskyLens(SingleScopeLens):

    propCache = list()
    keysCache = dict()
    lastPull = 0

    path = os.getenv("HOME") + "/.cleardarksky"
    prop = "http://cleardarksky.com//t/chart_prop00.txt"
    keys = "http://cleardarksky.com//t/chart_keys00.txt"

    class Meta:
        name = 'cleardarksky'
        description = 'Cleardarksky Lens'
        search_hint = 'Search Clear Dark Sky'
        icon = 'cleardarksky.svg'
        search_on_blank=False

    a_locations_category = IconViewCategory("Locations", "weather-clear-night-symbolic")
    b_gazing_category = ListViewCategory("Skygazing Conditions", "weather-clear-night-symbolic")
    
    def search(self, search, results):
        i = 0
        query = self.sky_query(search)
        if(os.path.isdir(self.path) is False):
            os.makedirs(self.path)
        for article in query:
            if(i == 25):
                #only show the top 25 results
                break
            os.system("convert -gravity SouthWest -crop 204x102+133+111 http://cleardarksky.com/c/" + article[0] + "csk.gif?1 ~/.cleardarksky/" + article[0] + ".png &")
            icon = self.path + "/" + article[0] + ".png"
            if(os.path.exists(self.path + "/" + article[0] + ".png") != True):
                # File isn't created yet, so use the full image instead
                icon = "http://cleardarksky.com/c/" + article[0] + "csk.gif?1"
            results.append("http://cleardarksky.com/c/" + article[0] + "key.html",
                           icon,
                           self.a_locations_category,
                           "text/html",
                           article[2] + ", " + article[1],
                           "Location",
                           "http://cleardarksky.com/c/" + article[0] + "key.html")
            i += 1
        if(len(query) > 0):
            article = query[0]
            data = self.fetchData(article[0])
            parsed = self.parseData(data)
            now = strftime("%Y-%m-%d %H:00:00", gmtime(time() + 60*60*parsed['utc']))
            
            year = strftime("%Y", gmtime(time() + 60*60*parsed['utc']))
            month = strftime("%m", gmtime(time() + 60*60*parsed['utc']))
            day = strftime("%d", gmtime(time() + 60*60*parsed['utc']))
            latitude = self.keysCache[str(article[0])][0]
            longitude = self.keysCache[str(article[0])][1]
            dayNight = self.calculateDayNight(int(year), int(month), int(day), int(parsed['utc']), latitude, longitude)
            if(float(parsed[now]['cloud']) < 7):
                weather_type = "few-clouds"
            else:
                weather_type = "clear"
                
            if dayNight == "day":
                icon = "/usr/share/icons/gnome/48x48/status/weather-" + weather_type + ".png"
            elif dayNight == "night":
                phase = str(self.calculateMoonPhase(int(year), int(month), int(day)))
                icon = "/usr/share/icons/gnome/scalable/status/weather-" + weather_type + "-night-" + str(phase) + ".svg"
            
            results.append("http://cleardarksky.com/c/" + article[0] + "key.html",
                           icon,
                           self.b_gazing_category,
                           "text/html",
                           "Cloud Cover:\t" + self.cloudIntToString(parsed[now]['cloud']) + "\n" + 
                           "Transparency:\t" + self.transIntToString(parsed[now]['trans']) + "\n" + 
                           "Seeing:\t\t" + self.seeingIntToString(parsed[now]['seeing']),
                           "Current Conditions",
                           "http://cleardarksky.com/c/" + article[0] + "key.html")
                           
            tommorow = dict()
            tommorow[0] = strftime("%Y-%m-%d 22:00:00", gmtime(time() + 60*60*parsed['utc'] + 2*60*60))
            tommorow[1] = strftime("%Y-%m-%d 23:00:00", gmtime(time() + 60*60*parsed['utc'] + 2*60*60))
            tommorow[2] = strftime("%Y-%m-%d 00:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            tommorow[3] = strftime("%Y-%m-%d 01:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            tommorow[4] = strftime("%Y-%m-%d 02:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            tommorow[5] = strftime("%Y-%m-%d 03:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            tommorow[6] = strftime("%Y-%m-%d 04:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            tommorow[7] = strftime("%Y-%m-%d 05:00:00", gmtime(time() + 60*60*parsed['utc'] + 26*60*60))
            
            avgClouds = 0
            avgTrans = 0
            avgSeeing = 0
            nItems = float(len(tommorow))
            for k, hour in tommorow.iteritems():
                avgClouds += float(parsed[hour]['cloud'])/nItems
                avgTrans += float(parsed[hour]['trans'])/nItems
                avgSeeing += float(parsed[hour]['seeing'])/nItems
            avgClouds = int(round(avgClouds))
            avgTrans = int(round(avgTrans))
            avgSeeing = int(round(avgSeeing))
            if(avgClouds < 7):
                weather_type = "few-clouds"
            else:
                weather_type = "clear"
                
            year = strftime("%Y", gmtime(time() + 60*60*parsed['utc'] + 2*60*60))
            month = strftime("%m", gmtime(time() + 60*60*parsed['utc'] + 2*60*60))
            day = strftime("%d", gmtime(time() + 60*60*parsed['utc'] + 2*60*60))
            phase = str(self.calculateMoonPhase(int(year), int(month), int(day)))

            if(len(phase) == 1):
                phase = "00" + phase
            elif(len(phase) == 2):
                phase = "0" + phase
            results.append("http://cleardarksky.com/c/" + article[0] + "key.html",
                           "/usr/share/icons/gnome/scalable/status/weather-" + weather_type + "-night-" + str(phase) + ".svg",
                           self.b_gazing_category,
                           "text/html",
                           "Cloud Cover:\t" + self.cloudIntToString(avgClouds) + "\n" + 
                           "Transparency:\t" + self.transIntToString(avgTrans) + "\n" + 
                           "Seeing:\t\t" + self.seeingIntToString(avgSeeing),
                           "Next Night's Conditions",
                           "http://cleardarksky.com/c/" + article[0] + "key.html")
        for article in query:
            if(i == 25):
                #only show the top 25 results
                break
    
    def fetchLocations(self):
        # According to cleardarksky.com, the list is generated around 3:00AM EST
        # I'm giving the server 1 hour to compute the new location data.  That should be more than
        # enough time to be sure that whatever is on the server is the latest
        try:
            t = gmtime(time() - 60*60*5)
            hour = int(str(strftime("%H", t)))
            newPull = int(strftime("%Y", t))*365 + int(strftime("%j", t))
            if((hour >= 4 and newPull > self.lastPull) or len(self.propCache) == 0):
                self.lastPull = newPull
                string = unicode(urllib2.urlopen(self.prop).read(), errors='ignore')
                self.propCache = list(string.split("\n"))
                string = unicode(urllib2.urlopen(self.keys).read(), errors='ignore')
                keys = list(string.split("\n"))
                self.keysCache = dict()
                for line in keys:
                    splitted = list(line.split('|'))
                    if len(splitted) == 4:
                        self.keysCache[str(splitted[0])] = (splitted[2], splitted[3])
                    else:
                        self.keysCache[str(splitted[0])] = (0, 0)
        except (IOError, KeyError, urllib2.URLError, urllib2.HTTPError):
            print "Error : Unable to search Clear Dark Sky"
            return []
    
    def fetchData(self, key):
        try:
            string = unicode(urllib2.urlopen("http://cleardarksky.com/txtc/" + key + "csp.txt").read(), errors='ignore')
            return list(string.split("\n"))
        except (IOError, KeyError, urllib2.URLError, urllib2.HTTPError):
            print "Error : Unable to search Clear Dark Sky"
            return []
    
    def cloudIntToString(self, val):
        val = int(val)
        if(val == 10):
            return "Clear"
        elif(val == 0):
            return "Overcast"
        return str((10 - val)*10) + "% covered"
    
    def transIntToString(self, val):
        val = int(val)
        if(val == 0):
            return "Too Cloudy"
        elif(val == 1):
            return "Bad"
        elif(val == 2):
            return "Poor"
        elif(val == 3):
            return "Average"
        elif(val == 4):
            return "Good"
        elif(val == 5):
            return "Excellent"
        return ""
    
    def seeingIntToString(self, val):
        if(val == "None"):
            return "Unknown"
        val = int(val)
        if(val == 0):
            return "Too Cloudy"
        elif(val == 1):
            return "Bad"
        elif(val == 2):
            return "Poor"
        elif(val == 3):
            return "Average"
        elif(val == 4):
            return "Good"
        elif(val == 1):
            return "Excellent"
        return ""
    
    def parseData(self, data):
        # returns a list where the values contain values from the given data
        parsed = dict()
        for line in data:
            if "#" in line:
                continue
            elif "UTC_offset" in line:
                splitted = line.split(' = ')
                parsed['utc'] = float(splitted[1])
            elif "00:00" in line:
                splitted = line.replace("\"", "").replace("(", "").replace(")", "").split(',')
                d = dict()
                d['time'] = splitted[0]
                d['cloud'] = splitted[1].replace(" ", "").replace("\t", "")
                d['trans'] = splitted[2].replace(" ", "").replace("\t", "")
                d['seeing'] = splitted[3].replace(" ", "").replace("\t", "")
                parsed[splitted[0]] = d
        return parsed
    
    def calculateMoonPhase(self, year, month, day):
        ret = 0
        r = year % 100
        r %= 19
        if (r>9):
            r -= 19
        r = ((r * 11) % 30) + int(month) + int(day)
        if (month<3):
            r += 2
        if (year<2000):
            r -= 4
        else:
            r -= 8.3
        r = floor(r+0.5)%30
        if (r < 0):
            ret = int(r+30)
        else:
            ret = int(r)
        ret *= 12
        ret = int(floor(ret/10.0)*10)
        return ret
        
        
    def calculateDayNight(self, year, month, day, localOffset, latitude, longitude):
        t = gmtime(time() + 60*60*localOffset)
        current = str(strftime("%H:%M:%S", t))
        setTime = "23:59:59"
        riseTime = "00:00:00"
        try:
            reader = libxml2.newTextReaderFilename("http://www.earthtools.org/sun/" + str(latitude) + "/" + str(longitude) + "/" + str(day) + "/" + str(month) + "/" + str(localOffset) + "/0")
            ret = reader.Read()
            while ret == 1:
                if reader.Name() == "sunset":
                    reader.Read()
                    setTime = str(reader.Value())
                    reader.Read()
                elif reader.Name() == "sunrise":
                    reader.Read()
                    riseTime = str(reader.Value())
                    reader.Read()
                ret = reader.Read()
        except:
            print "unable to open %s" % ("http://www.earthtools.org/sun/" + str(latitude) + "/" + str(longitude) + "/" + str(day) + "/" + str(month) + "/" + str(localOffset) + "/0")
        if (current > setTime or current < riseTime):
            return "night"
        else:
            return "day"
    
    def sky_query(self, search):
        results = list()
        self.fetchLocations()
        for line in self.propCache:
            splitted = list(line.split('|'))
            if(len(splitted) == 3):
                needle = search.lower()
                haystack = str(splitted[2].lower() + ", " + splitted[1].lower())
                pos = haystack.find(search.lower())
                if(pos >= 0):
                    row = (splitted[0], splitted[1], splitted[2], pos)
                    results.append(row)
        results.sort(lambda x,y: x[3] - y[3])
        return results
